# Generated by Django 4.0.10 on 2023-06-28 14:14

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("aa_discord_announcements", "0002_translation_updates"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pingtarget",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following groups …",
                related_name="discord_announcement_pingtarget_required_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following groups …",
                related_name="discord_announcement_webhook_required_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
    ]
