# Generated by Django 4.0.10 on 2023-09-22 17:41

# Django
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("aa_discord_announcements", "0003_fix_pseudo_plural_in_help_text"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="pingtarget",
            options={
                "default_permissions": (),
                "verbose_name": "Discord ping target",
                "verbose_name_plural": "Discord ping targets",
            },
        ),
    ]
