<p>
    {% if announcement_text %}
        {% if announcement_target.at_mention %}
            {{ announcement_target.at_mention }}

            <br><br>
        {% endif %}

        {{ announcement_text|linebreaksbr }}
    {% endif %}
</p>
