{% load django_bootstrap5 %}
{% load i18n %}

<form id="aa-discord-announcements-form" class="form-horizontal">
    {% csrf_token %}
    <fieldset class="aa-discord-announcements-form">
        <!-- announcement_target -->
        {% bootstrap_field form.announcement_target %}

        <!-- announcement_channel -->
        {% if webhooks_configured %}
            {% bootstrap_field form.announcement_channel  %}
        {% endif %}

        <!-- announcement_text -->
        {% bootstrap_field form.announcement_text %}

        <div class="form-group mt-3 clearfix">
            <div class=" col-sm-offset-2 col-sm-10 col-lg-offset-2 col-lg-10">
                <div class="aa-discord-announcements-form-message"></div>

                {% translate "Create announcement" as button_text %}
                {% bootstrap_button button_class="btn btn-primary" content=button_text name="createAnnouncementText" id="createAnnouncementText" %}
            </div>
        </div>
    </fieldset>
</form>
