@media all {
    .aa-discord-announcements-announcementtext > .card {
        position: sticky;
        top: 1rem;
    }

    .aa-discord-announcements form label .form-field-required {
        font-weight: 700;
    }
}
