{% extends "allianceauth/base-bs5.html" %}

{% load i18n %}

{% block page_title %}
    {% translate "Discord Announcements" %}
{% endblock %}

{% block header_nav_brand %}
    {% translate "Discord Announcements" %}
{% endblock header_nav_brand %}

{% block content %}
    <div class="aa-discord-announcements">
        <div class="aa-discord-announcements-header">
            {% block aa_discord_announcements_header %}{% endblock %}
        </div>

        <div class="aa-discord-announcements-body">
            {% block aa_discord_announcements_body %}{% endblock %}
        </div>

        <div class="aa-discord-announcements-footer mt-3 pt-3 border-top border-light">
            {% include "aa_discord_announcements/partials/footer/app-translation-footer.html" %}
        </div>
    </div>
{% endblock %}
